% this program simulates a double pendulum with point masses at the end of
% each link, the links themselves are massless.

clear all; close all; clc; echo off;

% various physical parameters
param.m1 = 0.2; param.m2 = 0.2;
param.L1 = 0.25; param.L2 = 0.25; param.g = 9.81;

Numtimelist = 2000;
tspan = linspace(0,20,Numtimelist)';
statevar0 = [pi/4; 0; 0; 0];
options = odeset('reltol',1e-9,'abstol',1e-9);

[tlist,statevarlist] = ode45(@doublependulumodefile,tspan,statevar0,options,param);

theta1list = statevarlist(:,1);
dtheta1list = statevarlist(:,2);
theta2list = statevarlist(:,3);
dtheta2list = statevarlist(:,4);

%%%% plot theta1, dtheta1, theta2, dtheta2

figure(1);
subplot(411); plot(tlist,theta1list); ylabel('\theta_1'); xlabel('t');
subplot(412); plot(tlist,theta2list); ylabel('\theta_2');  xlabel('t');
subplot(413); plot(tlist,dtheta1list); ylabel('d\theta_1/ dt'); xlabel('t');
subplot(414); plot(tlist,dtheta2list); ylabel('d\theta_2/ dt'); xlabel('t');

pause(3);

%%%% animation of the double pendulum

figure(2); L1 = param.L1; L2 = param.L2;
plot(0,0,'ko'); hold on;
hOA = plot([0 L1],[0 0],'r');
hold on;
hAB = plot([L1 L1+L2],[0 0],'b');
hA = plot(L1,0,'ro'); hB = plot(L1+L2,0,'bo');
axis equal; axis([-1.25*(L1+L2) 1.25*(L1+L2) -1.25*(L1+L2) 1.25*(L1+L2)]); 
set(hA,'markerfacecolor','r');
set(hB,'markerfacecolor','b');

for i = 1:Numtimelist
    xA = L1*cos(theta1list(i));
    yA = L1*sin(theta1list(i));
    xB = xA + L2*cos(theta1list(i)+theta2list(i));
    yB = yA + L2*sin(theta1list(i)+theta2list(i));
    set(hOA,'xdata',[0 xA],'ydata',[0 yA]);
    set(hA,'xdata',xA,'ydata',yA);
    set(hAB,'xdata',[xA xB],'ydata',[yA yB]);
    set(hB,'xdata',xB,'ydata',yB);
    pause(0.01);
end

%%%% computing total energy and showing that it remains very close to 
% a constant 
TE = zeros(Numtimelist,1);
for i = 1:Numtimelist
    theta1 = theta1list(i);
    dtheta1 = dtheta1list(i);
    theta2 = theta2list(i);
    dtheta2 = dtheta2list(i);
    vA = [-L1*sin(theta1)*dtheta1; L1*cos(theta1)*dtheta1; 0];
    vB = vA + [-L2*sin(theta1+theta2)*(dtheta1+dtheta2); ...
        L2*cos(theta1+theta2)*(dtheta1+dtheta2); 0];
    yA = L1*sin(theta1);
    yB = yA + L2*sin(theta1+theta2);
    KE = 0.5*param.m1*norm(vA)^2+0.5*param.m2*norm(vB)^2;
    PE = param.m1*param.g*yA+param.m2*param.g*yB;
    TE(i) = KE+PE;
end
figure(3);
plot(tlist,TE,'r'); ylabel('Total Energy');
xlabel('t')
change_in_energy = max(TE)-min(TE)
